/*
 Consola de Control de Lanzamiento
 Creado en 2014 por:
 Jesús Manuel Recuenco Andrés y Alejandro Amor García.
 Versión 11.
*/

// Declaración de las librerías 
#include <iostream>
#include <String.h>
#include <LiquidCrystal.h>
#include <SoftwareSerial.h>
#include <XBee.h>
#include <CControl.h>

using namespace std;

// Declaración de las constates
const int LT1 = 22;                                           // pin Led Test Ignitor-1 (luz verde)
const int LT2 = 24;                                           // pin Led Test Ignitor-2 (luz verde)
const int LT3 = 26;                                           // pin Led Test Ignitor-3 (luz verde)
const int LT4 = 28;                                           // pin Led Test Ignitor-4 (luz verde)
const int LA1 = 23;                                           // pin Led Armado Ignitor-1 (luz roja)
const int LA2 = 25;                                           // pin Led Armado Ignitor-2 (luz roja)
const int LA3 = 27;                                           // pin LEd Armado Ignitor-3 (luz roja)
const int LA4 = 29;                                           // pin Led Armado Ignitor-4 (luz roja)
const int LRV = 30;                                           // pin Led Ready (luz verde)
const int LRR = 31;                                           // pin Led Ready (luz roja)
const int ITR = 39;                                           // pin Interruptor Ready
const int ALM = 32;                                           // pin Alarma
const int IT1 = 36;                                           // pin Interruptor Test Ignitor-1
const int IT2 = 40;                                           // pin Interruptor Test Ignitor-2
const int IT3 = 44;                                           // pin Interruptor Test Ignitor-3
const int IT4 = 45;                                           // pin Interruptor Test Ignitor-4
const int IA1 = 34;                                           // pin Interruptor Armado Ignitor-1
const int IA2 = 38;                                           // pin Interruptor Armado Ignitor-2
const int IA3 = 42;                                           // pin Interruptor Armado Ignitor-3
const int IA4 = 46;                                           // pin Interruptor Armado Ignitor-4
const int ID1 = 41;                                           // pin Botón-pulsador disparo-1
const int ID2 = 43;                                           // pin Botón-pulsador disparo-2
const int SC1 = 37;                                           // pin 1 Selector de Terminal de Disparo II
const int SC2 = 35;                                           // pin 2 Selector de Terminal de Disparo III
const int SC3 = 33;                                           // pin 3 Selector de Terminal de Disparo IV

// Declaración de los objetos
LiquidCrystal lcd(12, 11, 5, 4, 3, 2);                        // Crea el Display LCD
XBee TerminalDisparo = XBee();                                // Crea la Terminal de Disparo
Int4p SelectorCD(SC1, SC2, SC3);                              // Crea el Selector de Terminal de Disparo
Int3p IntIgnitor1(IT1, IA1);                                  // Crea el Interruptor Ignitor-1
Int3p IntIgnitor2(IT2, IA2);                                  // Crea el Interruptor Ignitor-2
Int3p IntIgnitor3(IT3, IA3);                                  // Crea el Interruptor Ignitor-3
Int3p IntIgnitor4(IT4, IA4);                                  // Crea el Interruptor Ignitor-4
Int2p IntReady(ITR);                                          // Crea el Interruptor Ready
Int2p PulsadorD1(ID1);                                        // Crea el Pulsador para Disparo-1
Int2p PulsadorD2(ID2);                                        // Crea el Pulsador para Disparo-2
Led2c LedIgnitor1(LT1, LA1);                                  // Crea el Led Ignitor-1
Led2c LedIgnitor2(LT2, LA2);                                  // Crea el Led Ignitor-2
Led2c LedIgnitor3(LT3, LA3);                                  // Crea el Led Ignitor-3
Led2c LedIgnitor4(LT4, LA4);                                  // Crea el Led Ignitor-4
Led2c LedReady(LRV, LRR);                                     // Crea el Led Ready

// Definición de las variables globales
int  PosIntActu[4] = {0, 0, 0, 0};                            // Posición actual de los interruptores de Test/Armado
int  PosIntAnte[4] = {0, 0, 0, 0};                            // Posición anterior de los interruptores de Test/Armado
int  PosIntReady = 0;                                         // Posición de la llave de Ready
int  PosIntDisparo1 = 0;                                      // Posición del pulsador de disparo 1 (1=Pulsado, 0=Liberado)
int  PosIntDisparo2 = 0;                                      // Posición del pulsador de disparo 2 (1=Pulsado, 0=Liberado)
int  TerminalDisparoAnt = 0;                                  // Variable para Terminal de Disparo anterior
int  TerminalDisparoAct = 1;                                  // Variable para Terminal de Disparo actual(1=Terminal1, 2=Terminal-2, 3=Terminal-3, 4=Terminal-4)
int  Conexion;                                                // Variable para conexion con la Terminal de Disparo
int  Lanzado = 0;                                             // Variable de control para Lanzado
char comando[8] = {'\0','\0','\0','\0','\0','\0','\0','\0'};  // Array para el comando
char control = '\0';                                          // Caracter de control para el comando leído en el buffer
char crc = '\0';                                              // Caracter de control de integridad del comando
char rChsum = '\0';                                           // Caracter de control de integridad del comando recibido
char eChsum = '\0';                                           // Caracter de control de integridad del comando a enviar
char datosenvio[10] = {'\0','\0','\0','\0','\0','\0','\0','\0','\0','\0'}; // Trama de envío
char buffer[10] = {'\0','\0','\0','\0','\0','\0','\0','\0','\0','\0'}; // Trama del buffer
char respuesta[8] = {'\0','\0','\0','\0','\0','\0','\0','\0'}; // Array para construir comando recibido
char crecibido[8] = {'\0','\0','\0','\0','\0','\0','\0','\0'}; // Array para cadena de respuesta recibida
unsigned long d1;                                             // Variable para el control del instante de evnio del comando
unsigned long d2;                                             // Variable para el control del tiempo transcurrido tras enviar el comando
unsigned long t1;                                             // Variable para almacenar el instante de pulsación del Botón de disparo 1
unsigned long t2;                                             // Variable para almacenar el instante de pulsación del Botón de disparo 2
unsigned long intervalo;                                      // Variable para almacenar el intervalo entre t1 y t2
boolean pulsadoD1 = false;                                    // Booleano para saber si se ha pulsado el botón de disparo 1.
boolean pulsadoD2 = false;                                    // Booleano para saber si se ha pulsado el botón de disparo 2.
boolean Lanzamiento = false;                                  // Booleano para saber si hay lanzamiento
int cont1 = 0;                                                // Variable para continuidad en el ignitor 1.
int cont2 = 0;                                                // Variable para continuidad en el ignitor 2.
int cont3 = 0;                                                // Variable para continuidad en el ignitor 3.
int cont4 = 0;                                                // Variable para continuidad en el ignitor 4.
int Safety = 0;                                               // Variable para estado del interruptor Safety.

void setup(){
  Serial.begin(9600);                                         // Inicializa serial Display a 9600 bps
  Serial1.begin(9600);                                        // Inicializa serial Buffer a 9600 bps
  lcd.begin(16, 2);                                           // Definición del Dispaly LCD (16 columnas x 2 filas)
}

// ++++++++++++++++++++++++++++++++++++++++++++ BUCLE PRINCIPAL +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void loop(){
  Leer_IntStatus();
  if (TerminalDisparoAnt != TerminalDisparoAct) {
     Iniciar_sistema();
     Comprobar_Conexion();
     lcd.clear();
     lcd.print("PREPARADO.");
     TerminalDisparoAnt = TerminalDisparoAct;
  }
  if (PosIntAnte[0] != PosIntActu[0] | PosIntAnte[1] != PosIntActu[1] | PosIntAnte[2] != PosIntActu[2] | PosIntAnte[3] != PosIntActu[3]) { 
     if (PosIntAnte[0] != PosIntActu[0]) Comprobar_Ignitor1();
     if (PosIntAnte[1] != PosIntActu[1]) Comprobar_Ignitor2();
     if (PosIntAnte[2] != PosIntActu[2]) Comprobar_Ignitor3();
     if (PosIntAnte[3] != PosIntActu[3]) Comprobar_Ignitor4();
     lcd.clear();
     lcd.print("PREPARADO.");
  }
  if (PosIntReady == 1) {
     Preparar_lanzamiento();
     lcd.clear();
     lcd.print("PREPARADO.");
  }
}

// ++++++++++++++++++++++++++++++++++++++++++++ FUNCION: Leer_InStatus() ++++++++++++++++++++++++++++++++++++++++++++++++++
// Función que lee el estado de los interruptores en el panel de control (se excluyen los pulsadores de disparo)

void Leer_IntStatus(){
  PosIntActu[0] = IntIgnitor1.Posicion();
  PosIntActu[1] = IntIgnitor2.Posicion();
  PosIntActu[2] = IntIgnitor3.Posicion();
  PosIntActu[3] = IntIgnitor4.Posicion();
  PosIntReady = IntReady.Posicion();
  TerminalDisparoAct = SelectorCD.Posicion();
}

// ++++++++++++++++++++++++++++++++++++++++++++ FUNCION: Apagar_Leds() ++++++++++++++++++++++++++++++++++++++++++++++++++++
// Función que apaga todos los leds en el panel de la Consola de Control

void Apagar_Leds(){
  LedIgnitor1.PonerEstado('X');
  LedIgnitor2.PonerEstado('X');
  LedIgnitor3.PonerEstado('X');
  LedIgnitor4.PonerEstado('X');
  LedReady.PonerEstado('X');
}

// ++++++++++++++++++++++++++++++++++++++++++++ FUNCION: Iniciar_sistema() ++++++++++++++++++++++++++++++++++++++++++++++++
// Función para iniciar el sistema cada vez que se cambia de Terminal de Disparo o se reinicia la Consola de Control.

void Iniciar_sistema(){
  if (TerminalDisparoAnt != TerminalDisparoAct) {
     lcd.clear();
     lcd.print("INICIANDO       ");
     lcd.setCursor(0,1);
     lcd.print("SISTEMA...      ");
     delay(8000);
  }
  Apagar_Leds();
  delay(2000);
  Leer_IntStatus();
  PosIntAnte[0] = 0;
  PosIntAnte[1] = 0;
  PosIntAnte[2] = 0;
  PosIntAnte[3] = 0;
  if (PosIntAnte[0] != PosIntActu[0] | PosIntAnte[1] != PosIntActu[1] | PosIntAnte[2] != PosIntActu[2] | PosIntAnte[3] != PosIntActu[3] | PosIntReady == 1) {
     lcd.clear();
     lcd.print ("ERROR: APAGAR   ");
     lcd.setCursor(0,1);
     lcd.print ("INTERRUPTORES.  ");
     do {
        if (PosIntActu[0] == 0) {
           LedIgnitor1.PonerEstado('X');
           PosIntAnte[0] = PosIntActu[0];
          }else{
           LedIgnitor1.PonerEstado('R');
        }
        if (PosIntActu[1] == 0) {
           LedIgnitor2.PonerEstado('X');
           PosIntAnte[1] = PosIntActu[1];
          }else{
           LedIgnitor2.PonerEstado('R');
        }
        if (PosIntActu[2] == 0) {
           LedIgnitor3.PonerEstado('X');
           PosIntAnte[2] = PosIntActu[2];
          }else{
           LedIgnitor3.PonerEstado('R');
        }
        if (PosIntActu[3] == 0) {
           LedIgnitor4.PonerEstado('X');
           PosIntAnte[3] = PosIntActu[3];
        }else{
           LedIgnitor4.PonerEstado('R');
        }
        if (PosIntReady == 0) {
           LedReady.PonerEstado('X');
        }else{
           LedReady.PonerEstado('R');
        }
        delay(250);
        Leer_IntStatus();
        Apagar_Leds();
        delay(250);
     } while (PosIntAnte[0] != PosIntActu[0] | PosIntAnte[1] != PosIntActu[1] | PosIntAnte[2] != PosIntActu[2] | PosIntAnte[3] != PosIntActu[3] | PosIntReady == 1);
     Apagar_Leds();
     delay(2000);    
  }
}      

// ++++++++++++++++++++++++++++++++++++++++++++ FUNCION: Comprobar_Conexion() ++++++++++++++++++++++++++++++++++++++++++
// Función que comprueba la conexión con la Terminal de Disparo seleccionada en la consola

void Comprobar_Conexion() {
  lcd.clear();
  lcd.print ("TERMINAL: ");
  lcd.print (TerminalDisparoAct);
  lcd.setCursor(0,1);
  delay (1000);
  lcd.print ("CONEXION...      ");
  Envia_comando(strcpy(comando,"CCONECT"));
  lcd.setCursor(0,1);
  if (strcmp(crecibido,"CONN-OK") == 0) {
     lcd.print ("CONEXION OK.     ");
  }
  delay (2000);
}

// ++++++++++++++++++++++++++++++++++++++++++++ FUNCION: Comprobar_Seguridad() ++++++++++++++++++++++++++++++++++++++++++
// Función que comprueba la el estado del interruptor SAFE en la Terminal de Disparo seleccionada en la consola

void Comprobar_Seguridad() {
  lcd.clear();
  lcd.print ("TERMINAL: ");
  lcd.print (TerminalDisparoAct);
  lcd.setCursor(0,1);
  Envia_comando(strcpy(comando,"CSAFETY"));
  if (strcmp(crecibido,"SF-HIGH") == 0){
     lcd.print("SEGURIDAD ON.   ");
  }
  while (strcmp(crecibido,"SF-HIGH") == 0) {
    Envia_comando(strcpy(comando,"CSAFETY"));
    delay (1000);
  }
  lcd.print("SEGURIDAD OFF.  ");
  delay (2000);  
}
  
// ++++++++++++++++++++++++++++++++++++++++++++ FUNCION: Comprobar_Ignitor1() ++++++++++++++++++++++++++++++++++++++++++++
// Función que comprueba la posición del interruptor y el estado del ignitor 1 en la Termial de disparo seleccionada

void Comprobar_Ignitor1(){
  switch (PosIntActu[0]) {
         case 0:
               LedIgnitor1.PonerEstado('X');
               Envia_comando(strcpy(comando,"C1-XOFF"));
               break;
         case 1:
               if (PosIntReady == 0){
                  lcd.clear();
                  lcd.print ("TEST IGNITOR-1: ");
                  Envia_comando(strcpy(comando,"C1-TEST"));
                  lcd.setCursor(0,1);
                  if (strcmp(crecibido,"T1-HIGH") == 0) {
                     LedIgnitor1.PonerEstado('V');
                     lcd.print ("CONTINUIDAD OK. ");
                  }
                  if (strcmp(crecibido,"T1-DOWN") == 0){
                     LedIgnitor1.PonerEstado('X');
                     lcd.print ("SIN CONTINUIDAD.");
                  }
                  delay (2000);
                  break;
               }
         case 2:
               lcd.clear();
               lcd.print ("ARMA IGNITOR-1:      ");
               Envia_comando(strcpy(comando,"C1-ARMA"));
               lcd.setCursor(0,1);
               if (strcmp(crecibido,"T1-HIGH") == 0) {
                  LedIgnitor1.PonerEstado('R');
                  lcd.print ("ARMADO OK.      ");
               }
               if (strcmp(crecibido,"T1-DOWN") == 0){
                  LedIgnitor1.PonerEstado('X');
                  lcd.print ("SIN CONTINUIDAD.");
               }
               delay (2000);
               break;
  }
  PosIntAnte[0] = PosIntActu[0];
  lcd.clear();
}

// ++++++++++++++++++++++++++++++++++++++++++++ FUNCION: Comprobar_Ignitor2() ++++++++++++++++++++++++++++++++++++++++++++
// Función que comprueba la posición del interruptor y el estado del ignitor 2 en la Termial de disparo seleccionada

void Comprobar_Ignitor2(){
  switch (PosIntActu[1]) {
         case 0:
               LedIgnitor2.PonerEstado('X');
               Envia_comando(strcpy(comando,"C2-XOFF"));
               break;
         case 1:
               if (PosIntReady == 0){
                  lcd.clear();
                  lcd.print ("TEST IGNITOR-2: ");
                  Envia_comando(strcpy(comando,"C2-TEST"));
                  lcd.setCursor(0,1);
                  if (strcmp(crecibido,"T2-HIGH") == 0) {
                     LedIgnitor2.PonerEstado('V');
                     lcd.print ("CONTINUIDAD OK. ");
                  }
                  if (strcmp(crecibido,"T2-DOWN") == 0){
                     LedIgnitor2.PonerEstado('X');
                     lcd.print ("SIN CONTINUIDAD.");
                  }
                  delay (2000);
                  break;
               }
         case 2:
               lcd.clear();
               lcd.print ("ARMA IGNITOR-2:      ");
               Envia_comando(strcpy(comando,"C2-ARMA"));
               lcd.setCursor(0,1);
               if (strcmp(crecibido,"T2-HIGH") == 0) {
                  LedIgnitor2.PonerEstado('R');
                  lcd.print ("ARMADO OK.      ");
               }
               if (strcmp(crecibido,"T2-DOWN") == 0){
                  LedIgnitor2.PonerEstado('X');
                  lcd.print ("SIN CONTINUIDAD.");
               }
               delay (2000);
               break;
  }
  PosIntAnte[1] = PosIntActu[1];
  lcd.clear();
}

// ++++++++++++++++++++++++++++++++++++++++++++ FUNCION: Comprobar_Ignitor3() ++++++++++++++++++++++++++++++++++++++++++++
// Función que comprueba la posición del interruptor y el estado del ignitor 3 en la Termial de disparo seleccionada

void Comprobar_Ignitor3(){
  switch (PosIntActu[2]) {
         case 0:
               LedIgnitor3.PonerEstado('X');
               Envia_comando(strcpy(comando,"C3-XOFF"));
               break;
         case 1:
               if (PosIntReady == 0){
                  lcd.clear();
                  lcd.print ("TEST IGNITOR-3: ");
                  Envia_comando(strcpy(comando,"C3-TEST"));
                  lcd.setCursor(0,1);
                  if (strcmp(crecibido,"T3-HIGH") == 0) {
                     LedIgnitor3.PonerEstado('V');
                     lcd.print ("CONTINUIDAD OK. ");
                  }
                  if (strcmp(crecibido,"T3-DOWN") == 0){
                     LedIgnitor3.PonerEstado('X');
                     lcd.print ("SIN CONTINUIDAD.");
                  }
                  delay (2000);
                  break;
               }
         case 2:
               lcd.clear();
               lcd.print ("ARMA IGNITOR-3:      ");
               Envia_comando(strcpy(comando,"C3-ARMA"));
               lcd.setCursor(0,1);
               if (strcmp(crecibido,"T3-HIGH") == 0) {
                  LedIgnitor3.PonerEstado('R');
                  lcd.print ("ARMADO OK.      ");
               }
               if (strcmp(crecibido,"T3-DOWN") == 0){
                  LedIgnitor3.PonerEstado('X');
                  lcd.print ("SIN CONTINUIDAD.");
               }
               delay (2000);
               break;
  }
  PosIntAnte[2] = PosIntActu[2];
  lcd.clear();
}

// ++++++++++++++++++++++++++++++++++++++++++++ FUNCION: Comprobar_Ignitor4() ++++++++++++++++++++++++++++++++++++++++++++
// Función que comprueba la posición del interruptor y el estado del ignitor 4 en la Termial de disparo seleccionada

void Comprobar_Ignitor4(){
  switch (PosIntActu[3]) {
         case 0:
               LedIgnitor4.PonerEstado('X');
               Envia_comando(strcpy(comando,"C4-XOFF"));
               break;
         case 1:
               if (PosIntReady == 0){
                  lcd.clear();
                  lcd.print ("TEST IGNITOR-4: ");
                  Envia_comando(strcpy(comando,"C4-TEST"));
                  lcd.setCursor(0,1);
                  if (strcmp(crecibido,"T4-HIGH") == 0) {
                     LedIgnitor4.PonerEstado('V');
                     lcd.print ("CONTINUIDAD OK. ");
                  }
                  if (strcmp(crecibido,"T4-DOWN") == 0){
                     LedIgnitor4.PonerEstado('X');
                     lcd.print ("SIN CONTINUIDAD.");
                  }
                  delay (2000);
                  break;
               }
         case 2:
               lcd.clear();
               lcd.print ("ARMA IGNITOR-4:      ");
               Envia_comando(strcpy(comando,"C4-ARMA"));
               lcd.setCursor(0,1);
               if (strcmp(crecibido,"T4-HIGH") == 0) {
                  LedIgnitor4.PonerEstado('R');
                  lcd.print ("ARMADO OK.      ");
               }
               if (strcmp(crecibido,"T4-DOWN") == 0){
                  LedIgnitor4.PonerEstado('X');
                  lcd.print ("SIN CONTINUIDAD.");
               }
               delay (2000);
               break;
  }
  PosIntAnte[3] = PosIntActu[3];
  lcd.clear();
}

// ++++++++++++++++++++++++++++++++++++++++++++ FUNCION: Perparar_lanzamiento() +++++++++++++++++++++++++++++++++++++++++++
// Función de preparación para el lanzamiento y realización del mismo

void Preparar_lanzamiento() {
  Comprobar_Conexion();
  Leer_IntStatus();
  Lanzamiento = false;
  if (PosIntActu[0] == 2) Comprobar_Ignitor1();
  if (PosIntActu[1] == 2) Comprobar_Ignitor2();
  if (PosIntActu[2] == 2) Comprobar_Ignitor3();
  if (PosIntActu[3] == 2) Comprobar_Ignitor4();
  if (PosIntActu[0] == 2 | PosIntActu[1] == 2 | PosIntActu[2] == 2 | PosIntActu[3] == 2) {
      Comprobar_Seguridad();
      LedReady.PonerEstado('V');
      digitalWrite(ALM,HIGH);
      lcd.clear();
      lcd.print("**LANZAMIENTO** ");
      lcd.setCursor(0,1);
      lcd.print("** PREPARADO ** ");
      Envia_comando(strcpy(comando,"AL-HIGH"));
      Lanzamiento = true;
      pulsadoD1 = false;
      pulsadoD2 = false;
      do {
         if (Lanzamiento) {
            if (!pulsadoD1) {
               PosIntDisparo1 = PulsadorD1.Posicion();
               if (PosIntDisparo1 == 1) {
                  t1 = millis();
                  pulsadoD1 = true;
               }
            }
            if (!pulsadoD2) {
               PosIntDisparo2 = PulsadorD2.Posicion();
               if (PosIntDisparo2 == 1) {
               t2 = millis();
               pulsadoD2 = true;
               }
            }
            if (pulsadoD1 & pulsadoD2) {
               if (t1 > t2) {
                  intervalo = t1 - t2;
                 }else{
                  intervalo = t2 - t1;
               }
               if (intervalo <= 1000) {
                  // Realiza el disparo de los ignitores que estén armados y tengan continuidad
                  lcd.clear();
                  lcd.print("PULSADO OK.");
                  Envia_comando(strcpy(comando,"DLAUNCH"));
//                  lcd.print (crecibido);
//                  delay (2000);
                  LedReady.PonerEstado('R');
                  if (PosIntActu[0] == 2) LedIgnitor1.PonerEstado('X');
                  if (PosIntActu[1] == 2) LedIgnitor2.PonerEstado('X');
                  if (PosIntActu[2] == 2) LedIgnitor3.PonerEstado('X');
                  if (PosIntActu[3] == 2) LedIgnitor4.PonerEstado('X');
                  Lanzamiento = false;
                  while (PosIntReady == 1) {
                     if (strcmp(crecibido,"LANZAOK") == 0){
                        lcd.clear();
                        delay(500);
                        lcd.print("LANZAMIENTO OK. ");
                        lcd.setCursor(0,1);
                        lcd.print("PONER READY OFF.");
                        delay(500);
                       }else{
                        lcd.clear();
                        if (crecibido[1] == '1') LedIgnitor1.PonerEstado('R');
                        if (crecibido[2] == '1') LedIgnitor2.PonerEstado('R');
                        if (crecibido[3] == '1') LedIgnitor3.PonerEstado('R');
                        if (crecibido[4] == '1') LedIgnitor4.PonerEstado('R');
                        delay(250);
                        if (crecibido[1] == '1') LedIgnitor1.PonerEstado('X');
                        if (crecibido[2] == '1') LedIgnitor2.PonerEstado('X');
                        if (crecibido[3] == '1') LedIgnitor3.PonerEstado('X');
                        if (crecibido[4] == '1') LedIgnitor4.PonerEstado('X');
                        delay(250);
                        lcd.print("FALLO IGNITORES.");
                        lcd.setCursor(0,1);
                        lcd.print("PONER READY OFF.");
                        if (crecibido[1] == '1') LedIgnitor1.PonerEstado('R');
                        if (crecibido[2] == '1') LedIgnitor2.PonerEstado('R');
                        if (crecibido[3] == '1') LedIgnitor3.PonerEstado('R');
                        if (crecibido[4] == '1') LedIgnitor4.PonerEstado('R');
                        delay(250);
                        if (crecibido[1] == '1') LedIgnitor1.PonerEstado('X');
                        if (crecibido[2] == '1') LedIgnitor2.PonerEstado('X');
                        if (crecibido[3] == '1') LedIgnitor3.PonerEstado('X');
                        if (crecibido[4] == '1') LedIgnitor4.PonerEstado('X');
                        delay(250);
                     }
                     PosIntReady = IntReady.Posicion();
                  }
                  Apagar_Leds();
                  digitalWrite(ALM,LOW);
                  Envia_comando(strcpy(comando,"AL-DOWN"));
                  Iniciar_sistema();
               }else{
                  lcd.clear();
                  lcd.print("ERROR PULSACION ");
                  lcd.setCursor(0,1);
                  lcd.print("REPITA...       ");
                  do{
                  }while (PulsadorD1.Posicion() == 1 | PulsadorD2.Posicion() == 1);
               }
               pulsadoD1 = false;
               pulsadoD2 = false;
             }
         }
         PosIntReady = IntReady.Posicion();      
      } while (Lanzamiento & PosIntReady == 1);
      LedReady.PonerEstado('X');
    }else{
      LedReady.PonerEstado('R');
      lcd.clear();
      lcd.print("NO HAY IGNITORES");
      lcd.setCursor(0,1);
      lcd.print("ARMADOS.        ");
      while (PosIntReady == 1) {
             PosIntReady = IntReady.Posicion();
      }
      LedReady.PonerEstado('X');
      Lanzamiento = false;
      Iniciar_sistema();
    }
  if (Lanzamiento) {
     Lanzamiento = false;
     lcd.clear();
     lcd.print("LANZAMIENTO     ");
     lcd.setCursor(0,1);
     lcd.print("ANULADO.");
     digitalWrite(ALM,LOW);
     Envia_comando(strcpy(comando,"AL-DOWN"));
     LedReady.PonerEstado('X');
     delay(2000);
  }
}

// ++++++++++++++++++++++++++++++++++++++++++++ FUNCION: Envia_comando() +++++++++++++++++++++++++++++++++++++++++++
// Función que envía los comandos a la Terminal de Disparo

void Envia_comando(char * cadena){
     control = '\0';
     eChsum = '\0';
     rChsum = '\0';
     crc = '\0';
     for (int i = 0; i < 10; i++) datosenvio[i] = '\0';
     for (int i = 0; i < 8; i++) respuesta[i] = '\0';
  // Construye y envía la cadena de datos
     for (int i = 0; i < strlen(cadena); i++) eChsum ^= cadena[i];
     strcpy(datosenvio,"$");
     strcat(datosenvio,cadena);
     datosenvio[8] = eChsum;
     Serial1.write(datosenvio);
     delay(100);
  // Espera respuesta
     d1 = millis();
     do {
        d2 = millis();
        if (Serial1.available()>0){
           for (int i = 0; i < 10; i++) buffer[i] = '\0';
           Serial1.readBytes(buffer,10);
           control = buffer[0];
         }else{
           if (d2 - d1 > 5000) {
              lcd.setCursor(0,1);
              lcd.print("SIN CONEXION....");
              Serial1.write(datosenvio);
              delay(500);
              lcd.setCursor(0,1);
              lcd.print("                ");
              delay(500);
           }
        }            
     } while (control != '$');
  // Comprueba la respuesta recibida desde la Terminal de disparo.
     if (control == '$'){
        for (int i = 1; i <8; i++) respuesta[i-1] = buffer[i];
        crc = buffer[8];
        for (int i = 0; i < strlen(respuesta); i++) rChsum ^= respuesta[i];
        if (crc == rChsum) {
           strcpy(crecibido,respuesta);
         }else{
           strcpy(crecibido,"");
        }
        cadena = '\0';
     }
}


