/*
 Terminal de Disparo
 Creado en 2014 por:
 Jesús Manuel Recuenco Andrés y Alejandro Amor García
 Versión 7.
*/

// Declaración de las librerías 
#include <iostream>
#include <String.h>
#include <SoftwareSerial.h>
#include <XBee.h>

using namespace std;

int DS1 = 10;                                        // Pin para disparo del ignitor 1
int DS2 = 8;                                         // Pin para disparo del ignitor 2
int DS3 = 6;                                         // Pin para disparo del ignitor 3
int DS4 = 4;                                         // Pin para disparo del ignitor 4
int SEG = 13;                                        // Pin para el interruptor SAFE
int CN1 = 11;                                        // Pin para continuidad del ignitor 1
int CN2 = 9;                                         // Pin para continuidad del ignitor 2
int CN3 = 7;                                         // Pin para continuidad del ignitor 3
int CN4 = 5;                                         // Pin para continuidad del ignitor 4
int ALM = 12;                                        // Pin para ALARMA
char control;                                        // Caracter de control del comando
char rChsum;                                         // Caracter CRC calculado para el comando recibido
char eChsum;                                         // Caracter CRC calculado para el comando a enviar
char crc;                                            // Caracter para almacenar el CRC recibido
boolean fallo = false;                               // Booleano para el control de fallo en el disparo ignitores
char falloignitor[4] = {'0','0','0','0'};            // Array para indicar los ignitores fallidos
int armado[4] = {0,0,0,0};                           // Array para indicar los ignitores armados
char buffer[10];                                     // Array para almacenar la trama recibida
char crespuesta[8];                                  // Array para confeccionar la respuesta
char comandorecibido[10];                            // Array para tratamiento del comando recibido
char datosenvio[10];                                 // Array para enviar la respuesta


void setup() {
     Serial.begin(9600);
     Serial1.begin(9600);
     pinMode(DS1,OUTPUT);
     pinMode(DS2,OUTPUT);
     pinMode(DS3,OUTPUT);
     pinMode(DS4,OUTPUT);
     pinMode(SEG,INPUT);
     pinMode(CN1,INPUT);
     pinMode(CN2,INPUT);
     pinMode(CN3,INPUT);
     pinMode(CN4,INPUT);
     pinMode(ALM,OUTPUT);
     digitalWrite(ALM,LOW);
}

void loop(){
  do {
     if (Serial1.available()>0) {
        control = '\0';
        for (int i = 0; i<10; i++) buffer[i]='\0';
        Serial1.readBytes(buffer,10);
        Serial.println(buffer);
        control = buffer[0];
        delay(100);
     }
  } while (control != '$');
  if (control == '$'){
     // Inicializa variables
     eChsum = '\0';
     rChsum = '\0';
     crc = '\0';
     for (int i = 0; i<10; i++) comandorecibido[i]='\0';
     for (int i = 0; i<8; i++) crespuesta[i]='\0';
     for (int i = 0; i<10; i++) datosenvio[i]='\0';
     // Comprueba los datos recibidos
     for (int i = 1; i < 8; i++) comandorecibido[i-1] = buffer[i];
     crc = buffer[8];
     for (int i = 0; i < strlen(comandorecibido); i++) rChsum ^= comandorecibido[i];
     if (crc == rChsum){
        if (strcmp(comandorecibido,"CCONECT") == 0) strcpy(crespuesta,"CONN-OK");
        if (strcmp(comandorecibido,"AL-HIGH") == 0) digitalWrite(ALM,HIGH);
        if (strcmp(comandorecibido,"AL-DOWN") == 0) digitalWrite(ALM,LOW);
        if (strcmp(comandorecibido,"CSAFETY") == 0) {
           if (digitalRead(SEG) == 0){
              strcpy(crespuesta,"SF-HIGH");
            }else{
              strcpy(crespuesta,"SF-DOWN");
           }
        }
        if (strcmp(comandorecibido,"C1-XOFF") == 0) armado[0] = 0;
        if (strcmp(comandorecibido,"C1-TEST") == 0) {
           armado[0] = 0;
           if (digitalRead(CN1) == 0){
              strcpy(crespuesta,"T1-HIGH");
            }else{
              strcpy(crespuesta,"T1-DOWN");
           }
        }
        if (strcmp(comandorecibido,"C1-ARMA") == 0) {
           if (digitalRead(CN1) == 0){
              strcpy(crespuesta,"T1-HIGH");
              armado[0] = 1;             
            }else{
              strcpy(crespuesta,"T1-DOWN");
           }
        }
        if (strcmp(comandorecibido,"C2-XOFF") == 0) armado[1] = 0;
        if (strcmp(comandorecibido,"C2-TEST") == 0) {
           armado[1] = 0;
           if (digitalRead(CN2) == 0){
              strcpy(crespuesta,"T2-HIGH");
            }else{
              strcpy(crespuesta,"T2-DOWN");
           }
        }
        if (strcmp(comandorecibido,"C2-ARMA") == 0) {
           if (digitalRead(CN2) == 0){
              strcpy(crespuesta,"T2-HIGH");
              armado[1] = 1;             
            }else{
              strcpy(crespuesta,"T2-DOWN");
           }
        }
        if (strcmp(comandorecibido,"C3-XOFF") == 0) armado[2] = 0;
        if (strcmp(comandorecibido,"C3-TEST") == 0) {
           armado[2] = 0;
           if (digitalRead(CN3) == 0){
              strcpy(crespuesta,"T3-HIGH");
            }else{
              strcpy(crespuesta,"T3-DOWN");
           }
        }
        if (strcmp(comandorecibido,"C3-ARMA") == 0) {
           if (digitalRead(CN3) == 0){
              strcpy(crespuesta,"T3-HIGH");
              armado[2] = 1;             
            }else{
              strcpy(crespuesta,"T3-DOWN");
           }
        }
        if (strcmp(comandorecibido,"C4-XOFF") == 0) armado[3] = 0;
        if (strcmp(comandorecibido,"C4-TEST") == 0) {
           armado[3] = 0;
           if (digitalRead(CN4) == 0){
              strcpy(crespuesta,"T4-HIGH");
            }else{ 
              strcpy(crespuesta,"T4-DOWN");
           }
        }
        if (strcmp(comandorecibido,"C4-ARMA") == 0) {
           if (digitalRead(CN4) == 0){
              strcpy(crespuesta,"T4-HIGH");
              armado[3] = 1;             
            }else{
              strcpy(crespuesta,"T4-DOWN");
           }
        }
        if (strcmp(comandorecibido,"DLAUNCH") == 0) {
           for (int i=0; i<4; i++) falloignitor[i] = '0';
           if (armado[0] == 1) digitalWrite(DS1,HIGH);
           if (armado[1] == 1) digitalWrite(DS2,HIGH);
           if (armado[2] == 1) digitalWrite(DS3,HIGH);
           if (armado[3] == 1) digitalWrite(DS4,HIGH);
           delay(500);                   // Tiempo de encendido de los ignitores
           digitalWrite(DS1,LOW);
           digitalWrite(DS2,LOW);
           digitalWrite(DS3,LOW);
           digitalWrite(DS4,LOW);
           fallo = false;
           falloignitor[0] = '0';
           falloignitor[1] = '0';
           falloignitor[2] = '0';
           falloignitor[3] = '0';
           Serial.print("armado[0]=");
           Serial.println(armado[0]);
           Serial.print("armado[1]=");
           Serial.println(armado[1]);
           Serial.print("armado[2]=");
           Serial.println(armado[2]);
           Serial.print("armado[3]=");
           Serial.println(armado[3]);
           Serial.print(" ");
           if (armado[0] == 1 && digitalRead(CN1) == 0){
              fallo = true;
              falloignitor[0] = '1';
           }
           if (armado[1] == 1 && digitalRead(CN2) == 0){
              fallo = true;
              falloignitor[1] = '1';
           }
           if (armado[2] == 1 && digitalRead(CN3) == 0){
              fallo = true;
              falloignitor[2] = '1';
           }
           if (armado[3] == 1 && digitalRead(CN4) == 0){
              fallo = true;
              falloignitor[3] = '1';
           }
           for (int i=0; i<4; i++) armado[i] = 0;
           if (fallo){
              strcpy(crespuesta,"L");
              for (int i=0; i<4; i++) crespuesta[i+1] = falloignitor[i];
              strcat(crespuesta,"-F");
              fallo = false;
             }else{
              strcpy(crespuesta,"LANZAOK");
           }
        }
        for (int i = 0; i < strlen(crespuesta); i++) eChsum ^= crespuesta[i];
        strcpy(datosenvio,"$");
        strcat(datosenvio,crespuesta);
        datosenvio[8]=eChsum;
        Serial1.write(datosenvio);
      }
     control = '\0';
  }
}
